/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : AbstractClassBrowser
   date       : 08.07.98
   description: an abstract base class for the implementation of a
                class browser, that can be displayed as part of the
                codepad panel.

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.Frame;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import javax.swing.Action;

import Utils.ResourceUtil;
import Utils.FrameUtil;
import Messages.Message;
import Messages.ProgressMessage;
import Messages.ClearProgressMessage;
import Messages.TerminateMessage;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public abstract class AbstractClassBrowser extends Observable {

 // instance variable declarations
  protected final ResourceBundle resources;
  protected final Properties project;

  // constructors
  /** Should be called from a subclass. */
  public AbstractClassBrowser(ResourceBundle resources,
                              Properties project) {
    this.resources = resources;
    this.project   = project;
  } // end constructor AbstractClassBrowser

  // class methods
  /**
   * Returns an instance of a derived class of AbstractClassBrowser.
   *
   * @param clazz The class name.
   * @param resources Resource bundle containing GUI labels, etc.
   * @param project Project related properties.
   */
  public static AbstractClassBrowser getInstance(String clazz,
                  ResourceBundle resources, Properties project) {
    Object[] objects;
    Class[] classes;
    Class c;
    Constructor constr;
    AbstractClassBrowser browser = null;

    if (clazz == null) return null;
    objects    = new Object[2];
    objects[0] = resources;
    objects[1] = project;
    classes    = new Class[2];
    classes[0] = java.util.ResourceBundle.class;
    classes[1] = java.util.Properties.class;
    try {
      c       = Class.forName(clazz);
      constr  = c.getConstructor(classes);
      browser = (AbstractClassBrowser)constr.newInstance(objects);
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
    } catch (IllegalAccessException excep) {
       excep.printStackTrace();
    } catch (InstantiationException excep) {
       excep.printStackTrace();
    } catch (NoSuchMethodException excep) {
       excep.printStackTrace();
    } catch (InvocationTargetException excep) {
       excep.printStackTrace();
    }
    return browser;
  } // end getInstance

  // methods
  /** Returns a new instance of a class browser view panel. */
  public abstract Component getViewPanel();

  /** Returns an array containing all project files. */
  protected File[] getProjectFiles() {
    int numFiles  = Integer.parseInt(project.getProperty(Const.NUM_FILES));
    Vector vector = new Vector(numFiles);
    File[] files;

    for (int i=0; i<numFiles; i++) {
      String name = project.getProperty(Const.FILE_PREFIX + i);

      if (name == null) {
        System.err.println("AbstractClassBrowser: Project file number wrong.");
        break;
      } // end if
      if (name.startsWith("file:"))
        vector.addElement(new File(ResourceUtil.urlToFile(name)));
      else
        vector.addElement(new File(name));
      // end if
    } // end for
    numFiles = vector.size();
    files = new File[numFiles];
    for (int i=0; i<numFiles; i++) files[i] = (File)vector.elementAt(i);
    return files;
  } // end getProjectFiles

  /** Returns the project directory. */
  protected String getProjectDirectory() {
    // This method can both read files and urls
    String str;

    str = project.getProperty(Const.PROJECT_DIR);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Project directory not spezified");
    if (str.startsWith("file:/")) str = ResourceUtil.urlToFile(str);
    return str;
  } // end getProjectDirectory

  /** Marks the current project as changed. */
  protected void setProjectChanged() {
    project.put(Const.PROJECT_CHANGED,"" + true);
  } // end setProjectChanged

  /** Posts a message to all observers of this observable. */
  public void postMessage(Message msg) {
    setChanged();
    notifyObservers(msg);
  } // end postMessage

  /**
   * Notifies all Observers from a file scan.
   *
   * @param text Description.
   * @param file The file which is in progress.
   * @param cur Number of the current file.
   * @param max Number of files to scan.
   */
  protected void notifyScan(String text, File file, int cur, int max) {
    postMessage(new ProgressMessage(text,file,cur,max,Message.LOW));
 } // end notifyScan

  /** Notifies all Observers from finishing the file scan. */
  protected void notifyEndOfScan(boolean success) {
    postMessage(new ClearProgressMessage(Message.LOW));
    postMessage(new TerminateMessage(success));
 } // end notifyEndOfScan

  /**
   * Updates the class browsers modell.
   *
   * @param frame A frame that can be used as parent for dialogs.
   */
  public void update(Frame frame) {
    update(frame,getProjectFiles());
  } // end update

  /**
   * Updates the class browsers modell.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @param files The source files of the project.
   */
  public abstract void update(Frame frame, File[] files);

  /**
   * Show an editor window containing the specified file and
   * set the caret to the specified location.
   *
   * @param frame The parent frame of the component invoking this method.
   * @param file The file to display.
   * @param line The line to set the caret to.
   */
  protected void showLocation(Frame frame, File file, int line) {
    CodepadPanel cpp;
    Action[] actions;
    Action newWin = null;

    if (frame == null) {
      Toolkit.getDefaultToolkit().beep();
      System.err.println("AbstractClassBrowser: Hosting frame not found.");
      return;
    } // end if
    cpp = (CodepadPanel)FrameUtil.contains(frame,CodepadPanel.class);
    if (cpp == null) {
      Toolkit.getDefaultToolkit().beep();
      System.err.println("AbstractClassBrowser: CodepadPanel not found.");
      return;
    } // end if
    actions  = cpp.getActions();
    for (int i=0; i<actions.length; i++) {
      Action a = actions[i];

      if (Const.NEWWINDOW_ACTION.equals(a.getValue(Action.NAME))) {
        newWin = a;
        break;
      } // end if
    } // end for
    if (newWin == null) {
      Toolkit.getDefaultToolkit().beep();
      System.err.println("AbstractClassBrowser: Action \"" +
                         Const.NEWWINDOW_ACTION + " not found.");
      return;
    } // end if
    newWin.actionPerformed(new ActionEvent(this,
      ActionEvent.ACTION_PERFORMED,line + " " +
      file.getAbsolutePath()));
  } // end showLocation

} // end AbstractClassBrowser

/* ******************************************************************
   end of file
*********************************************************************/
