/* ******************************************************************

   class    : AbstractCompiler
   date     : 27.07.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.Component;
import java.awt.Label;
import java.awt.Frame;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Properties;
import java.util.Vector;
import java.util.Observable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import Utils.ResourceUtil;
import Messages.Message;
import Messages.ProgressMessage;
import Messages.ClearProgressMessage;
import Messages.TerminateMessage;
import Messages.ClearErrorMessage;

/**
 * An abstract base class for compiler plugins.
 *
 * @author puehringer edgar
 */
public abstract class AbstractCompiler extends Observable {

  // instance variable declarations
  protected final ResourceBundle resources;
  protected final Properties project;
  /** True if a ProgressMessage had been sent. */
  private boolean progressMessageSent = false;

  // constructors
  /** Create a new AbstractCompiler class. */
  public AbstractCompiler(ResourceBundle resources,Properties project) {
    this.resources = resources;
    this.project   = project;
  } // end constructor AbstractCompiler

  // class methods
  /**
   * Returns an instance of a derived class of AbstractCompiler.
   *
   * @param clazz The class name.
   * @param resources Resource bundle containing GUI labels, etc.
   * @param project Project related properties.
   */
  public static AbstractCompiler getInstance(String clazz,
                  ResourceBundle resources, Properties project) {
    Object[] objects;
    Class[] classes;
    Class c;
    Constructor constr;
    AbstractCompiler compiler = null;

    if (clazz == null) return null;
    objects    = new Object[2];
    objects[0] = resources;
    objects[1] = project;
    classes    = new Class[2];
    classes[0] = java.util.ResourceBundle.class;
    classes[1] = java.util.Properties.class;
    try {
      c        = Class.forName(clazz);
      constr   = c.getConstructor(classes);
      compiler = (AbstractCompiler)constr.newInstance(objects);
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
    } catch (IllegalAccessException excep) {
       excep.printStackTrace();
    } catch (InstantiationException excep) {
       excep.printStackTrace();
    } catch (NoSuchMethodException excep) {
       excep.printStackTrace();
    } catch (InvocationTargetException excep) {
       excep.printStackTrace();
    }
    return compiler;
  } // end getInstance

  // methods
  /** Compile a single file. */
  public void compile(File file) {
    File[] files = new File[1];

    files[0] = file;
    compile(files);
  } // end compile

  /** Compile an array of files. */
  public abstract void compile(File[] files);

  /** Compile all files that have been changed and that depend from. */
  public void make() {
    File[] files = make(getProjectFiles());

    if (files != null) setObjectFiles(files);
  } // end make

  /**
   * Compile all files that have been changed and that depend from.
   *
   * @return An array containing the object files.
   */
  public abstract File[] make(File[] files);

  /** Compile all project files. */
  public void build() {
    File[] files = build(getProjectFiles());

    if (files != null) setObjectFiles(files);
  } // end build

  /**
   * Compile all project files.
   *
   * @return An array containing the object files.
   */
  public abstract File[] build(File[] files);

  /**
   * Return copyright information.
   */
  public Component getInfo() {
    return new Label(this.getClass().getName());
  } // end getInfo

  /**
   * Displays a settings dialog.
   */
  public void settings(Frame parent) {
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      false,"Method should not be called");
  } // end settings

  /**
   * Returns true, if the settings method is available.
   */
  public boolean hasSettings() {
    Class clazz    = this.getClass();
    Class[] params = new Class[1];
    boolean found  = true;
    Method method;

    try {
      params[0] = Class.forName("java.awt.Frame");
      method    = clazz.getDeclaredMethod("settings",params);
    } catch (NoSuchMethodException excep) {
      found = false;
    } catch (SecurityException excep) {
      excep.printStackTrace();
      found = false;
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
      found = false;
    }
    return found;
  } // end hasSettings

  /** Returns an array containing all project files. */
  protected File[] getProjectFiles() {
    int numFiles  = Integer.parseInt(project.getProperty(Const.NUM_FILES));
    Vector vector = new Vector(numFiles > 0 ? numFiles : 1);
    File[] files;

    for (int i=0; i<numFiles; i++) {
      String name = project.getProperty(Const.FILE_PREFIX + i);

      if (name == null) {
        System.err.println("AbstractCompiler: Project file number wrong.");
        break;
      } // end if
      if (name.startsWith("file:"))
        vector.addElement(new File(ResourceUtil.urlToFile(name)));
      else
        vector.addElement(new File(name));
      // end if
    } // end for
    numFiles = vector.size();
    files = new File[numFiles];
    for (int i=0; i<numFiles; i++) files[i] = (File)vector.elementAt(i);
    return files;
  } // end getProjectFiles

  /**
   * Writes an array containing object files to the project properties.
   */
  protected void setObjectFiles(File[] files) {
    String str;
    int i;

    for (i=0; i<files.length; i++) {
      str = ResourceUtil.fileToURL(files[i].getAbsolutePath());
      project.put(Const.OBJECT_PREFIX + i,str);
    } // end for
    project.put(Const.NUM_OBJECT,"" + files.length);
    // erase old object references
    i = files.length;
    while (project.containsKey(Const.OBJECT_PREFIX + i)) {
      project.remove(Const.OBJECT_PREFIX + i);
      i++;
    } // end while
  } // end setObjectFiles

  /**
   * Returns an array containing all extensions of source files.
   */
  protected String[] getExtensions() {
    String str;

    str = project.getProperty(Const.SOURCE_EXT);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Extension for source files not found");
    return ResourceUtil.splitToWords(str);
  } // end getExtensions

  /**
   * Returns an array containing all extensions of object files.
   */
  protected String[] getObjectExtensions() {
    String str;

    str = project.getProperty(Const.OBJECT_EXT);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Extension for object files not found");
    return ResourceUtil.splitToWords(str);
  } // end getExtensions

  /** Returns the project directory. */
  protected String getProjectDirectory() {
    // This method can both read files and urls
    String str;

    str = project.getProperty(Const.PROJECT_DIR);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Project directory not spezified");
    if (str.startsWith("file:/")) str = ResourceUtil.urlToFile(str);
    return str;
  } // end getProjectDirectory

  /** Marks the current project as changed. */
  protected void setProjectChanged() {
    project.put(Const.PROJECT_CHANGED,""+true);
  } // end setProjectChanged

  /** Posts a message to all observers of this observable. */
  public void postMessage(Message msg) {
    setChanged();
    notifyObservers(msg);
  } // end postMessage

  /**
   * Notifies all Observers from compiling a file.
   *
   * @param text Description.
   * @param file The file which is in progress.
   * @param cur Number of the current file.
   * @param max Number of files to compile.
   */
  protected void notifyCompile(String text, File file, int cur, int max) {
    postMessage(new ProgressMessage(text,file,cur,max,Message.NORMAL));
    progressMessageSent = true;
  } // end notifyCompile

  /** Notifies all Observers from finishing compiling. */
  protected void notifyStartOfCompile() {
    postMessage(new ClearErrorMessage());
 } // end notifyStartOfCompile

  /** Notifies all Observers from finishing compiling. */
  protected void notifyEndOfCompile(boolean success) {
    if (progressMessageSent) {
      postMessage(new ClearProgressMessage(Message.NORMAL));
      progressMessageSent = false;
    } // end if
    postMessage(new TerminateMessage(success));
 } // end notifyEndOfCompile

} // end AbstractCompiler

/* ******************************************************************
   end of file
*********************************************************************/
