/* ******************************************************************

   class    : AbstractLinker
   date     : 26.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.Component;
import java.awt.Label;
import java.awt.Frame;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Properties;
import java.util.Vector;
import java.util.Observable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import Utils.ResourceUtil;
import Messages.Message;
import Messages.ProgressMessage;
import Messages.ClearProgressMessage;
import Messages.TerminateMessage;
import Messages.ClearErrorMessage;

/**
 * An abstract base class for linker plugins.
 * A derived class can have a method settings(java.awt.Frame)
 * which is automatically recognized by the environment. The
 * purpose of this method is to specify linker settings.
 *
 * @author puehringer edgar
 */
public abstract class AbstractLinker extends Observable {

  // instance variable declarations
  protected final ResourceBundle resources;
  protected final Properties project;
  /** True if a ProgressMessage had been sent. */
  private boolean progressMessageSent = false;

  // constructors
  /** Create a new AbstractLinker class. */
  public AbstractLinker(ResourceBundle resources, Properties project) {
    this.resources = resources;
    this.project   = project;
  } // end constructor AbstractLinker

  // class methods
  /**
   * Returns an instance of a derived class of AbstractLinker.
   *
   * @param clazz The class name.
   * @param resources Resource bundle containing GUI labels, etc.
   * @param project Project related properties.
   */
  public static AbstractLinker getInstance(String clazz,
                  ResourceBundle resources, Properties project) {
    Object[] objects;
    Class[] classes;
    Class c;
    Constructor constr;
    AbstractLinker linker = null;

    if (clazz == null) return null;
    objects    = new Object[2];
    objects[0] = resources;
    objects[1] = project;
    classes    = new Class[2];
    classes[0] = java.util.ResourceBundle.class;
    classes[1] = java.util.Properties.class;
    try {
      c      = Class.forName(clazz);
      constr = c.getConstructor(classes);
      linker = (AbstractLinker)constr.newInstance(objects);
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
    } catch (IllegalAccessException excep) {
       excep.printStackTrace();
    } catch (InstantiationException excep) {
       excep.printStackTrace();
    } catch (NoSuchMethodException excep) {
       excep.printStackTrace();
    } catch (InvocationTargetException excep) {
       excep.printStackTrace();
    }
    return linker;
  } // end getInstance

  // methods
  /**
   * Invoke the linker for the current project.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @see getProjectFiles
   * @see getParameters
   * @see notifyTermination
   */
  public void link(Frame frame) {
    File[] files = link(frame,getProjectFiles());

    if (files != null) setExecutableFiles(files);
  } // end link

  /**
   * Invoke the linker for the current project.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @param files The object files of the project to link.
   * @return An array containing the executable files.
   * @see getProjectFiles
   * @see getParameters
   * @see notifyTermination
   */
  public abstract File[] link(Frame frame, File[] files);

  /**
   * Return copyright information.
   */
  public Component getInfo() {
    return new Label(this.getClass().getName());
  } // end getInfo

  /** Displays a settings dialog. */
  public void settings(Frame parent) {
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      false,"Method should not be called");
  } // end settings

  /** Returns true, if the settings method is available. */
  public boolean hasSettings() {
    String[] params = new String[1];

    params[0] = "java.awt.Frame";
    return hasMethod("settings",params);
  } // end hasSettings

  /** Returns true, if the specified method is available. */
  private boolean hasMethod(String method, String[] paramClasses) {
    Class clazz    = this.getClass();
    Class[] params = new Class[paramClasses.length];
    boolean found  = true;
    Method m;

    try {
      for (int i=0; i<params.length; i++)
        params[i] = Class.forName(paramClasses[i]);
      // end for
      m = clazz.getDeclaredMethod(method,params);
    } catch (NoSuchMethodException excep) {
      found = false;
    } catch (SecurityException excep) {
      excep.printStackTrace();
      found = false;
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
      found = false;
    }
    return found;
  } // end hasMethod

  /**
   * Returns an array containing all object files of this project.
   */
  protected File[] getProjectFiles() {
    int numFiles  = Integer.parseInt(project.getProperty(Const.NUM_OBJECT));
    Vector vector = new Vector(numFiles);
    File[] files;

    for (int i=0; i<numFiles; i++) {
      String name = project.getProperty(Const.OBJECT_PREFIX + i);

      if (name == null) {
        System.err.println("AbstractLinker: Object file number wrong.");
        break;
      } // end if
      vector.addElement(new File(ResourceUtil.urlToFile(name)));
    } // end for
    numFiles = vector.size();
    files = new File[numFiles];
    for (int i=0; i<numFiles; i++) files[i] = (File)vector.elementAt(i);
    return files;
  } // end getProjectFiles

  /**
   * Writes an array containing executable files to the
   * project properties.
   */
  protected void setExecutableFiles(File[] files) {
    String str;
    int i;

    for (i=0; i<files.length; i++) {
      str = ResourceUtil.fileToURL(files[i].getAbsolutePath());
      project.put(Const.EXEC_PREFIX + i,str);
    } // end for
    project.put(Const.NUM_EXEC,"" + files.length);
    // erase old executable references
    i = files.length;
    while (project.containsKey(Const.EXEC_PREFIX + i)) {
      project.remove(Const.EXEC_PREFIX + i);
      i++;
    } // end while
  } // end setExecutableFiles

  /** Returns an array containing all extensions of object files. */
  protected String[] getExtensions() {
    String str;

    str = project.getProperty(Const.OBJECT_EXT);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Extension for object files not found");
    return ResourceUtil.splitToWords(str);
  } // end getExtensions

  /** Returns an array containing all extensions of executable files. */
  protected String[] getExecExtensions() {
    String str;

    str = project.getProperty(Const.EXEC_EXT);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Extension for executable files not found");
    return ResourceUtil.splitToWords(str);
  } // end getExecExtensions

  /** Returns the project directory. */
  protected String getProjectDirectory() {
    // This method can both read files and urls
    String str;

    str = project.getProperty(Const.PROJECT_DIR);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Project directory not spezified");
    if (str.startsWith("file:/")) str = ResourceUtil.urlToFile(str);
    return str;
  } // end getProjectDirectory

  /** Marks the current project as changed. */
  protected void setProjectChanged() {
    project.put(Const.PROJECT_CHANGED,""+true);
  } // end setProjectChanged

  /**
   * Protects the environment from access violations caused
   * by the linking process.
   */
  protected void protectEnvironment(boolean enabled) {
    SwitchableSecurityManager sm;

    sm = (SwitchableSecurityManager)System.getSecurityManager();
    sm.setEnabled(enabled);
  } // end protectEnvironment

  /** Posts a message to all observers of this observable. */
  public void postMessage(Message msg) {
    setChanged();
    notifyObservers(msg);
  } // end postMessage

  /**
   * Notifies all Observers from linking a file.
   *
   * @param text Description.
   * @param file The file which is in progress.
   * @param cur Number of the current file.
   * @param max Number of files to link.
   */
  protected void notifyLink(String text, File file, int cur, int max) {
    postMessage(new ProgressMessage(text,file,cur,max,Message.NORMAL));
    progressMessageSent = true;
  } // end notifyLink

  /** Notifies all Observers from finishing linking. */
  protected void notifyEndOfLink(boolean success) {
    if (progressMessageSent) {
      postMessage(new ClearProgressMessage(Message.NORMAL));
      progressMessageSent = false;
    } // end if
    postMessage(new TerminateMessage(success));
 } // end notifyEndOfLink

} // end AbstractLinker

/* ******************************************************************
   end of file
*********************************************************************/
