/* ******************************************************************

   class    : AppletTagParser
   date     : 19.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package AppletView;

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Parses the first applet tag of a html stream.
 *
 * @author puehringer edgar
 */
public class AppletTagParser {

  // instance variable declarations
  private InputStreamReader isr = null;
  private BufferedReader br     = null;
  private StreamTokenizer tokenizer;
  private AppletTag tag    = new AppletTag();
  private boolean tagValid = false;

  // constructors
  /** Create a new AppletTagParser instance. */
  public AppletTagParser(InputStream in) {
    isr = new InputStreamReader(in);
    br  = new BufferedReader(isr);
    init(br);
  } // end constructor AppletTagParser

  /** Create a new AppletTagParser instance. */
  public AppletTagParser(Reader in) {
    init(in);
  } // end constructor AppletTagParser

  // class methods
/* DEBUG ON */
  /** Only for debugging purposes. */
  public static void main(String[] args) throws IOException {
    AppletTagParser parser = new AppletTagParser(System.in);

    try {
      System.out.println("" + parser.parse());
    } catch (AppletTagParsingException excep) {
      System.err.println("Line number: " + parser.lineno());
      throw (IOException)excep.fillInStackTrace();
    }
  } // end main
/* DEBUG OFF */

  // methods
  private final void init(Reader in) {
    tokenizer = new StreamTokenizer(in);
    tokenizer.resetSyntax();
    // this are nearly the default settings; I had to reset the syntax
    // to turn the number parsing off.
    tokenizer.wordChars('A','Z');
    tokenizer.wordChars('a','z');
    tokenizer.wordChars('\u00A0','\u00FF');
    tokenizer.whitespaceChars('\u0000','\u0020');
    tokenizer.quoteChar('\'');
    tokenizer.quoteChar('"');
    tokenizer.eolIsSignificant(false);
    tokenizer.slashSlashComments(false);
    tokenizer.slashStarComments(false);
    tokenizer.eolIsSignificant(false);

    tokenizer.wordChars('0','9');
    tokenizer.wordChars('-','.');
  } // end init

  /** Parse from a reader. */
  public AppletTag parse() throws IOException {
    int type;

    try {
      while ((type = tokenizer.nextToken()) != StreamTokenizer.TT_EOF) {
        if (type == (int)'<') {
          type = tokenizer.nextToken();
          if ((type == StreamTokenizer.TT_WORD) &&
              tokenizer.sval.toLowerCase().equals("applet")) {
            tag.startLine = tokenizer.lineno();
            parseApplet();
            break; // break after first applet tag
          } // end if
        } // end if
      } // end while
    } finally {
      if (br != null) try { br.close(); } catch (IOException e) {}
      if (isr != null) isr.close();
    }
    return tagValid ? tag : null;
  } // end parse

  /** Get the line number of the scanner. */
  public final int lineno() {
    return tokenizer.lineno();
  } // end lineno

  private final void parseApplet() throws IOException {
    int type;
    String str;

    tag.width  = -1;
    tag.height = -1;
    while ((type = tokenizer.nextToken()) != '>') {
      tokenizer.pushBack();
      str = readWord();
      if (str.equals("codebase")) {
        if (tag.codebase != null)
          error("Duplicate identifier \"" + str + '"');
        // end if
        expect('=');
        tag.codebase = readLiteral();
      } else if (str.equals("code")) {
        if (tag.code != null)
          error("Duplicate identifier \"" + str + '"');
        // end if
        expect('=');
        tag.code = readLiteral();
      } else if (str.equals("width")) {
        if (tag.width != -1)
          error("Duplicate identifier \"" + str + '"');
        // end if
        expect('=');
        tag.width = readInteger();
      } else if (str.equals("height")) {
        if (tag.height != -1)
          error("Duplicate identifier \"" + str + '"');
        // end if
        expect('=');
        tag.height = readInteger();
      } else {
        error("Unknown identifier \"" + str + '"');
      } // end if
    } // end while
    if (tag.code == null)
      error("\"code\" expected.");
    else if (tag.width == -1)
      error("\"width\" expected.");
    else if (tag.height == -1)
      error("\"height\" expected.");
    // end if
    while ((type = tokenizer.nextToken()) != StreamTokenizer.TT_EOF) {
      if (type == (int)'<') {
        type = tokenizer.nextToken();
        if ((type == StreamTokenizer.TT_WORD) &&
            tokenizer.sval.toLowerCase().equals("param")) {
          parseParam();
        } else if (type == (int)'/') {
          type = tokenizer.nextToken();
          if ((type == StreamTokenizer.TT_WORD) &&
              tokenizer.sval.toLowerCase().equals("applet")) {
            expect('>');
            tagValid     = true;
            tag.numLines = tokenizer.lineno() - tag.startLine + 1;
            break;
          } // end if
        } // end if
      } // end if
    } // end while
    if (!tagValid) error("</APPLET> tag expected.");
  } // end parseApplet

  private final void parseParam() throws IOException {
    int type;
    String str;
    String name  = null;
    String value = null;

    while ((type = tokenizer.nextToken()) != '>') {
      tokenizer.pushBack();
      str = readWord();
      if (str.equals("name")) {
        if (name != null)
          error("Duplicate identifier \"" + str + '"');
        // end if
        expect('=');
        name = readWord();
      } else if (str.equals("value")) {
        if (value != null)
          error("Duplicate identifier \"" + str + '"');
        // end if
        expect('=');
        value = readLiteral();
      } else {
        error("Unknown identifier \"" + str + '"');
      } // end if
    } // end while
    if (name == null)
      error("\"name\" expected.");
    else if (value == null)
      error("\"value\" expected.");
    // end if
    tag.params.put(name,value);
  } // end parseParam

  private final String readWord() throws IOException {
    int type = tokenizer.nextToken();

    if (type != StreamTokenizer.TT_WORD)
      error("Identifier expected.");
    // end if
    return tokenizer.sval.toLowerCase();
  } // end readWord

  private final String readLiteral() throws IOException {
    int type = tokenizer.nextToken();

    switch (type) {
      case StreamTokenizer.TT_WORD:
      case (int)'"':
        return tokenizer.sval;
      default:
        error("Quoted or unquoted string literal expected.");
    } // end switch
    return tokenizer.sval;
  } // end readLiteral

  private final int readInteger() throws IOException {
    int type = tokenizer.nextToken();
    String str;
    int value;

    if (type == StreamTokenizer.TT_WORD) str = tokenizer.sval;
      else str = "no number";
    try {
      value = Integer.parseInt(str);
    } catch (NumberFormatException excep) {
      error("Integer expected.");
      value = 0; // to avoid not initialized error
    }
    return value;
  } // end readInteger

  private final void expect(char ch) throws IOException {
    if (tokenizer.nextToken() != (int)ch) {
      error("'" + ch + "' expected.");
    } // end if
  } // end expect

  private final void expect(String word) throws IOException {
    if ((tokenizer.nextToken() != StreamTokenizer.TT_WORD) ||
        !tokenizer.sval.equals(word)) {
      error('"' + word + "\" expected.");
    } // end if
  } // end expect

  private final void error(String str) throws IOException {
    throw new AppletTagParsingException(str);
  } // end error

} // end AppletTagParser

/* ******************************************************************
   end of file
*********************************************************************/
