/* ******************************************************************

   class    : DefaultMacroprocessor
   date     : 02.09.98
   compiler : javac (jdk 1.2)
   modified : 21.4.99 - Connecting lines with backslash is no longer
                        supported.

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.ActionEvent;
import java.text.SimpleDateFormat;

import CustCtrl.BaseDialog;
import CustCtrl.ActionAdapter;
import CustCtrl.ImageCanvas;
import Utils.ResourceUtil;
import Utils.StringUtil;
import Messages.ErrorMessage;
import Messages.LocatableErrorMessage;
import Streams.PreprocMainFilter;

/**
 * A macroprocessor that produces source code templates.
 *
 * @author puehringer edgar
 */
public class DefaultMacroprocessor extends AbstractMacroprocessor {

  // class variable declarations
  /** Key to the current source code template file resource. */
  private final static String CUR_FILE = "preproc.currentFile";
  /** Key to the open file dialog title. */
  public final static String OPENDLG_TITLE = "preproc.openDlg.title";
  /** Key to the custom dialog title. */
  public final static String CUSTDLG_TITLE = "preproc.customDlg.title";
  /** Key to the custom dialog button labels. */
  public final static String CUSTDLG_BUTTONS = "preproc.customDlg.buttons";
  /** Key to the custom dialog select button label. */
  public final static String CUSTDLG_SELBUTTON = "preproc.customDlg.selButton";

  // instance variable declarations
  /** The hashtable that holds the identifiers (defines). */
  protected Properties identifiers;
  /** A incrementing number that saves the order of identifiers. */
  private int identNr;
  /** True if the generation of a template has been canceled. */
  private boolean canceled;
  /** The current frame that is the parent of dialogs to display. */
  private Frame curFrame;
  /** The current scanned file. */
  private File curFile;
  /** Used from a action adapter to get the current index. */
  private int curIndex;
  /** The current line in the template file. */
  private int curLine;

  // constructors
  /** Create a new DefaultMacroprocessor class. */
  public DefaultMacroprocessor(ResourceBundle resources, Properties project) {
    super(resources,project);
    identifiers = new Properties();
    identNr     = Integer.MIN_VALUE;
  } // end constructor DefaultMacroprocessor

  // class methods
/* DEBUG ON */
  public static void main(String[] args) {
    Frame frame = new Frame();
    DefaultMacroprocessor preproc;
    ResourceBundle res;
    Properties proj = new Properties();
    Reader reader;
    char[] buff  = new char[4096];
    int nch;

    if (args.length < 1) {
      System.out.println("Resource file must be specified.");
      System.exit(1);
    } // end if
    res     = ResourceBundle.getBundle(args[0],java.util.Locale.getDefault());
    proj.put("templateExtension",".tpl");
    preproc = new DefaultMacroprocessor(res,proj);
    reader = preproc.getTemplate(frame);
    if (reader != null) {
      try {
        while ((nch = reader.read(buff,0,buff.length)) != -1) {
          System.out.print(new String(buff,0,nch));
        } // end while
        reader.close();
      } catch (IOException excep) {
        excep.printStackTrace();
      }
    } // end if
    System.exit(0);
  } // end main
/* DEBUG OFF */

  // methods
  /**
   * Returns a Reader to read a file template from.  If the creation
   * of the template was canceled, then return null.
   *
   * @param parent A frame that can be used as parent for dialogs.
   */
  public Reader getTemplate(Frame parent) {
    File file;
    String str;

    curFrame = parent;
    do {
      file = getFile(curFrame);
      if (file == null) return null;
    } while(!file.exists());
    str = file.getParent();
    if (str != null) setTemplateDirectory(str);
    return getTemplate(parent,file);
  } // end getTemplate

  /**
   * Returns a Reader to read a file template from.  If the creation
   * of the template was canceled, then return null.
   *
   * @param parent A frame that can be used as parent for dialogs.
   * @param file A file to read the template from.
   */
  public Reader getTemplate(Frame parent, File file) {
    FileReader fr;
    Macroprocessor filter;
    Date date;
    SimpleDateFormat formatter;
    String str;
    char[] aCh;
    String[] aStr;
    CharArrayWriter caw = new CharArrayWriter();
    char[] buff         = new char[4096];
    int nch;

    curFrame = parent;
    try {
      fr = new FileReader(file);
    } catch (FileNotFoundException fnfe) {
      return null;
    }
    canceled = false;
    curFile  = file;
    // create the default defines
    identifiers.clear();
    identifiers.put("__LINE__","1");
    curLine = 0;
    identifiers.put("__FILE__",curFile.getAbsolutePath());
    date      = new Date();
    formatter = new SimpleDateFormat("MMM dd yyyy");
    identifiers.put("__DATE__",formatter.format(date));
    formatter = new SimpleDateFormat("HH:mm:ss");
    identifiers.put("__TIME__",formatter.format(date));
    identifiers.put("__USER__",System.getProperty("user.name"));
    str = project.getProperty(Const.SOURCE_EXT);
    try {
      identifiers.put("__EXT__",ResourceUtil.splitToWords(str)[0]);
    } catch (NullPointerException excep) {
      excep.printStackTrace();
    }
    str = project.getProperty(Const.COMPILER_VERSION);
    if (str != null) identifiers.put("__COMPILER__",str);
    str = project.getProperty(Const.CUR_APPLET);
    if (str != null) {
      String path = getProjectDirectory();

      str = ResourceUtil.urlToFile(str);
      if (str.startsWith(path)) {
        str = str.substring(path.length());
        if (str.startsWith(File.separator)) str = str.substring(1);
      } // end if
      identifiers.put("__APPLET__",str);
    } // end if
    aCh = System.getProperty("java.version").toCharArray();
    for (int i=0; i<aCh.length; i++)
      if (!Character.isDigit(aCh[i])) aCh[i] = ' ';
    // end for
    str  = (new String(aCh)).trim();
    aStr = ResourceUtil.splitToWords(str);
    if (aStr.length >= 2) {
      str = "__VERSION" + aStr[0] + '_' + aStr[1];
      identifiers.put(str,System.getProperty("java.version"));
      if (aStr.length >= 3) {
        str += (' ' + aStr[2]);
        identifiers.put(str,System.getProperty("java.version"));
      } // end if
    } // end if
    filter = new Macroprocessor(fr);
    try {
      while ((nch = filter.read(buff,0,buff.length)) != -1) {
        caw.write(buff,0,nch);
      } // end while
    } catch (IOException excep) {
      excep.printStackTrace();
      canceled = true;
    }
    if (canceled) return null;
    buff = caw.toCharArray();
    // clean up
    try { filter.close(); } catch (IOException excep) {}
    filter = null;
    caw    = null;
    // store the current template file
    str = ResourceUtil.fileToURL(curFile.getAbsolutePath());
    if (!str.equals(project.getProperty(CUR_FILE))) {
      project.put(CUR_FILE,str);
      setProjectChanged();
    } // end if
    System.gc();
    return new CharArrayReader(buff);
  } // end getTemplate

  /**
   * Returns the expansion of a macro defined in the last call
   * to getTemplate or null if it hadn't been defined.
   *
   * @param name The name of the macro.
   */
  public String getMacro(String name) {
    return identifiers.getProperty(name);
  } // end getMacro

  /**
   * Returns true if the given character equals the escape character for
   * preprocessor directives.
   */
  public boolean isPreprocEsc(char ch) {
    return Macroprocessor.isPreprocEsc(ch);
  } // end isPreprocEsc

  private final File getFile(Frame frame) {
    String str = project.getProperty(CUR_FILE);
    String name,path;
    File file;
    FileDialog dlg;

    if (str == null) {
      name = "*" + getExtensions()[0];
      path = getTemplateDirectory();
    }
    else {
      file = new File(ResourceUtil.urlToFile(str));
      name = file.getName();
      path = file.getParent();
    } // end if
    str = ResourceUtil.getResStringOrKey(resources,OPENDLG_TITLE);
    dlg = new FileDialog(frame,str,FileDialog.LOAD);
    dlg.setDirectory(path);
    dlg.setFile(name);
    dlg.show();
    name = dlg.getFile();
    if (name != null) {
      path = dlg.getDirectory();
      if (path == null) return new File(name);
        else return new File(path,name);
    } // end if
    return null;
  } // end getFile

  protected void postWarning(String text, int line) {
    LocatableErrorMessage em;

    em = new LocatableErrorMessage(ErrorMessage.WARNING,text,
                                   new File(getProjectDirectory()),
                                   curFile,line,null);
    postMessage(em);
  } // end postWarning

  protected void postError(String text, int line) {
    LocatableErrorMessage em;

    em = new LocatableErrorMessage(ErrorMessage.ERROR,text,
                                   new File(getProjectDirectory()),
                                   curFile,line,null);
    postMessage(em);
  } // end postError

  protected void requestUserInput(Hashtable ht) {
    CustomDlg dlg;

    if (!canceled) {
      dlg = new CustomDlg(curFrame,ht);
      dlg.show();
      if (dlg.isCanceled()) canceled = true;
        else dlg.readResults();
    } // end if
  } // end requestUserInput

  // inner classes
  class Macroprocessor extends PreprocMainFilter {

    // instance variable declarations
    /** Holds Booleans that are true if the if-stat. is in the else-part. */
    private Stack ifStack = new Stack();
    /** Holds the if-level that turned off the output. */
    private int turnOffLevel;
    /** Holds the identifiers that will be queried from the user. */
    private Hashtable pendingIdents = new Hashtable();

    // constructors
    public Macroprocessor(Reader in) {
      super(in,4096);
      identNr = 0;
    } // end constructor Macroprocessor

    /** Checks if a identifier had been defined. */
    protected boolean isDefined(String ident) {
      Enumeration enum;
      Object key,val;

      if (pendingIdents.containsKey(ident)) {
        requestUserInput(pendingIdents);
        enum = pendingIdents.keys();
        while (enum.hasMoreElements()) {
          key = enum.nextElement();
          val = ((DlgItem)pendingIdents.get(key)).initVal;
          if (val instanceof Boolean) {
            if (((Boolean)val).booleanValue()) {
              if (!identifiers.containsKey(key))
                identifiers.put(key,"");
              // end if
            }
            else {
              identifiers.remove(key);
            } // end if
          } else if (val instanceof File) {
            String str = ((File)val).getAbsolutePath();

            str = StringUtil.relativePath(str,getProjectDirectory());
            identifiers.put(key,str);
          } else {
            Utils.Assert.assert(Utils.Assert.ENABLED &&
              (val instanceof String));
            if (((String)val).equals("")) identifiers.remove(key);
              else identifiers.put(key,val);
          } // end if
        } // end while
        pendingIdents.clear();
      } // end if
      if (identifiers.getProperty(ident) == null) return false;
        else return true;
    } // end isDefined

    /** Changes identifiers to previous stored define values. */
    protected String evalIdentifier(String ident) {
      int i = getLineNumber();

      if (isDefined(ident)) {
        if (curLine != i) {
          identifiers.put("__LINE__","" + i);
          curLine = i;
        } // end if
        return identifiers.getProperty(ident);
      }
      else {
        return ident;
      } // end if
    } // end evalIdentifier

    /** Parse a macroprocessor command line. */
    protected void processCmdLine(String str) {
      String[] s = ResourceUtil.splitToWords(str);
      String content;

      if (s.length < 1) return;
      content = str.substring(s[0].length());
      content = content.trim();
      switch (s[0].charAt(0)) {
        case 'd':
          if (s[0].equals("define")) parseDefine(content);
            else unknown(s[0]);
        break;
        case 'u':
          if (s[0].equals("undef")) parseUndef(content);
            else unknown(s[0]);
        break;
        case 'e':
          if (s[0].equals("error")) parseError(content);
            else if (s[0].equals("endif")) parseEndif(content);
              else if (s[0].equals("else")) parseElse(content);
                else if (s[0].equals("elif")) nosupport(s[0]);
                  else unknown(s[0]);
        break;
        case 'p':
          if (s[0].equals("pragma")) parsePragma(content);
            else unknown(s[0]);
        break;
        case 'i':
          if (s[0].equals("ifdef")) parseIfdef(content);
            else if (s[0].equals("ifndef")) parseIfndef(content);
              else if (s[0].equals("include")) nosupport(s[0]);
                else if (s[0].equals("if")) nosupport(s[0]);
                  else unknown(s[0]);
        break;
        case 'l':
          if (s[0].equals("line")) nosupport(s[0]);
            else unknown(s[0]);
        break;
        default:
          unknown(s[0]);
      } // end switch
    } // end processCmdLine

    private final void parseDefine(String str) {
      if (!isOutputEnabled()) return;

      String[] s = ResourceUtil.splitToWords(str);

      if ((s.length < 1) || !isIdentifier(s[0])) {
        expect("Identifier");
      }
      else {
        str = str.substring(s[0].length());
        str = processTokenSeq(str.trim());
        identifiers.put(s[0],str);
        pendingIdents.remove(s[0]);
      } // end if
    } // end parseDefine

    private final void parseUndef(String str) {
      if (!isOutputEnabled()) return;

      String[] s = ResourceUtil.splitToWords(str);

      if ((s.length < 1) || !isIdentifier(s[0])) {
        expect("Identifier");
      } else if (s.length > 1) {
        expect("End of line");
      } else {
        identifiers.remove(s[0]);
        pendingIdents.remove(s[0]);
      } // end if
    } // end parseUndef

    private final void parseError(String str) {
      if (!isOutputEnabled()) return;

      str = processTokenSeq(str);
      postError(str,getLineNumber());
    } // end parseError

    private final void parseEndif(String str) {
      if (ifStack.size() < 1) {
        postError("Misplaced endif",getLineNumber());
      }
      else {
        if (!str.equals("")) expect("End of line");
        ifStack.pop();
        if (!isOutputEnabled() &&
            (turnOffLevel == ifStack.size())) {
          enableOutput(true);
        } // end if
      } // end if
    } // end parseEndif

    private final void parseElse(String str) {
      if (ifStack.size() < 1) {
        postError("Misplaced else",getLineNumber());
        return;
      } // end if
      if (!str.equals("")) expect("End of line");
      if (((Boolean)ifStack.pop()).booleanValue()) {
        postError("Misplaced else",getLineNumber());
      }
      else {
        if (isOutputEnabled() ||
            (ifStack.size() == turnOffLevel)) {
          enableOutput(!isOutputEnabled());
          turnOffLevel = ifStack.size();
        } // end if
      } // end if
      ifStack.push(new Boolean(true));
    } // end parseElse

    private final void parsePragma(String str) {
      if (!isOutputEnabled()) return;

      String[] s = ResourceUtil.splitToWords(str);
      String content;

      if (s.length < 1) {
        expect("Subcommand");
        return;
      } // end if
      content = str.substring(s[0].length());
      content = content.trim();
      if (s[0].equals("unquote")) {
        if ((s.length < 2) || !isIdentifier(s[1])) {
          expect("Identifier");
        } else if (s.length > 2) {
          expect("End of line");
        } else {
          str = identifiers.getProperty(s[1]);
          if (str != null)
            identifiers.put(s[1],StringUtil.replaceEscapeSeq(str));
          // end if
        } // end if
      } else if (s[0].equals("quote")) {
        if ((s.length < 2) || !isIdentifier(s[1])) {
          expect("Identifier");
        } else if (s.length > 2) {
          expect("End of line");
        } else {
          str = identifiers.getProperty(s[1]);
          if (str != null)
            identifiers.put(s[1],StringUtil.convertToLiteral(str));
          // end if
        } // end if
      } else if (s[0].equals("getstring")) {
        if ((s.length < 2) || !isIdentifier(s[1])) expect("Identifier");
          else getString(s[1],content);
      } else if (s[0].equals("getbool")) {
        if ((s.length < 2) || !isIdentifier(s[1])) expect("Identifier");
          else getBool(s[1],content);
      } else if (s[0].equals("getfile")) {
        if ((s.length < 2) || !isIdentifier(s[1])) expect("Identifier");
          else getFile(s[1],content);
      } // end if
    } // end parsePragma

    private final void parseIfdef(String str) {
      parseIf(str,false);
    } // end parseIfdef

    private final void parseIfndef(String str) {
      parseIf(str,true);
    } // end parseIfndef

    private final void parseIf(String str, boolean inv) {
      boolean takeIf;

      if (!isIdentifier(str)) {
        expect("Single identifier");
        takeIf = inv;
      }
      else {
        takeIf = (isDefined(str));
        if (inv) takeIf = !takeIf;
      } // end if
      if (isOutputEnabled() && !takeIf) {
        enableOutput(false);
        turnOffLevel = ifStack.size();
      } // end if
      ifStack.push(new Boolean(false));
    } // end parseIf

    private final void getString(String ident, String line) {
      String initVal = identifiers.getProperty(ident);

      if (initVal == null) initVal = "";
      line = line.substring(ident.length()).trim();
      pendingIdents.put(ident,new DlgItem(line,initVal));
    } // end getString

    private final void getBool(String ident, String line) {
      Boolean initVal = new Boolean(identifiers.containsKey(ident));

      line = line.substring(ident.length()).trim();
      pendingIdents.put(ident,new DlgItem(line,initVal));
    } // end getBool

    private final void getFile(String ident, String line) {
      String str = identifiers.getProperty(ident);
      File initVal;

      if (str == null) {
        initVal = new File(getProjectDirectory());
      }
      else {
        initVal = new File(str);
        if (!initVal.isAbsolute())
          initVal = new File(getProjectDirectory(),str);
        // end if
      } // end if
      line = line.substring(ident.length()).trim();
      pendingIdents.put(ident,new DlgItem(line,initVal));
    } // end getFile

    private final void unknown(String str) {
      postError("Unknown macroprocessor control \"" + str +
                "\".",getLineNumber());
    } // end unknown

    private final void nosupport(String str) {
      postWarning("\"" + str + "\" not supported.",getLineNumber());
    } // end nosupport

    private final void expect(String str) {
      postError(str + " expected.",getLineNumber());
    } // end nosupport

  } // end Macroprocessor

  /** Dialog item for the automatic dialog generation. */
  class DlgItem {

    // instance variable declarations
    String desc;
    Object initVal;
    int nr;

    // constructors
    public DlgItem(String desc, Object initVal) {
      this.desc    = desc;
      this.initVal = initVal;
      this.nr      = identNr++;
    } // end constructor DlgItem
  } // end DlgItem

  /** A dialog that requests defines from the user. */
  class CustomDlg extends BaseDialog {

    // instance variable declarations
    DlgItem[] items;
    Component[] comps;

    // constructors
    /** Create a new CustomDlg object. */
    public CustomDlg(Frame parent, Hashtable ht) {
      super(parent,true);

      Panel cp          = getContentPanel();
      Panel panel       = new Panel();
      Enumeration enum  = ht.keys();
      int i,j;
      DlgItem tmp;
      Label label;
      Button btn;
      String title;
      Image image;

      cp.setLayout(new BorderLayout());
      items = new DlgItem[ht.size()];
      comps = new Component[ht.size()];
      // copy the dialog items to an array
      i = 0;
      while (enum.hasMoreElements()) {
        items[i++] = (DlgItem)ht.get(enum.nextElement());
      } // end while
      // sort the array
      for (i=0; i<items.length; i++) {
        for (j=1; j<items.length; j++) {
          if (items[j - 1].nr > items[j].nr) {
            tmp          = items[j - 1];
            items[j - 1] = items[j];
            items[j]     = tmp;
          } // end if
        } // end for
      } // end for
      // create the components
      for (i=0; i<comps.length; i++) {
        if (items[i].initVal instanceof Boolean) {
          boolean b = ((Boolean)items[i].initVal).booleanValue();

          comps[i] = new Checkbox(items[i].desc,b);
        } else if (items[i].initVal instanceof File) {
          String str = ((File)items[i].initVal).getAbsolutePath();
          TextField tf;

          str = StringUtil.relativePath(str,getProjectDirectory());
          tf = new TextField(str,15);
          tf.setEditable(false);
          comps[i] = tf;
        } else {
          Utils.Assert.assert(Utils.Assert.ENABLED &&
            (items[i].initVal instanceof String));
          String str   = (String)items[i].initVal;
          TextField tf = new TextField(str,15);

          comps[i] = tf;
        } // end if
      } // end for
      // do the layout
      title = getMacro("__TITLE__");
      if (title == null) {
        title = ResourceUtil.getResStringOrKey(resources,CUSTDLG_TITLE);
      } // end if
      setTitle(title);
      title   = getMacro("__IMAGE__");
      if (title == null) {
        image = null;
      }
      else {
        title = "" + new File(curFile.getParent(),title);
        image = getToolkit().getImage(title);
      } // end if
      createButtons(ResourceUtil.getResStringOrKey(resources,CUSTDLG_BUTTONS));
      title = getMacro("__BUTTON__");
      if ((title != null) && (buttonVector.size() > 0)) {
        Button b = (Button)buttonVector.elementAt(0);

        b.setActionCommand(b.getLabel());
        b.setLabel(title);
      } // end if
      GridBagLayout gridbag          = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      panel.setLayout(gridbag);
      constraints.anchor     = GridBagConstraints.WEST;
      constraints.insets     = new Insets(10,6,0,6);
      constraints.weightx    = 2.0;
      constraints.weighty    = 2.0;
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      constraints.gridheight = 1;
      for (i=0; i<comps.length; i++) {
        if (i == (comps.length - 1)) {
          constraints.gridheight = GridBagConstraints.REMAINDER;
          constraints.insets     = new Insets(10,6,10,6);
        } else if (i == (comps.length - 2)) {
          constraints.gridheight = GridBagConstraints.RELATIVE;
        } // end if
        if (comps[i] instanceof TextField) {
          TextField tf  = (TextField)comps[i];
          Insets insets = constraints.insets;

          constraints.gridwidth    = 1;
          constraints.insets       = (Insets)constraints.insets.clone();
          constraints.insets.right = 0;
          label = new Label(items[i].desc);
          gridbag.setConstraints(label,constraints);
          panel.add(label);
          if (tf.isEditable()) {
            constraints.insets     = insets;
            constraints.gridwidth  = GridBagConstraints.REMAINDER;
          }
          else {
            constraints.gridwidth = GridBagConstraints.RELATIVE;
          } // end if
          gridbag.setConstraints(tf,constraints);
          panel.add(tf);
          if (!tf.isEditable()) {
            constraints.weightx   = 1.0;
            constraints.insets    = insets;
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            btn = new Button(ResourceUtil.getResStringOrKey(resources,CUSTDLG_SELBUTTON));
            curIndex = i;
            btn.addActionListener(
              new ActionAdapter() {

                int index = curIndex;

                public void actionPerformed(ActionEvent e) {
                  selectFile(index);
                } // end actionPerformed

              }
            );
            gridbag.setConstraints(btn,constraints);
            panel.add(btn);
            constraints.weightx = 2.0;
          } // end if
        }
        else {
          gridbag.setConstraints(comps[i],constraints);
          panel.add(comps[i]);
        } // end if
      } // end for
      cp.add(panel,BorderLayout.CENTER);
      if (image != null) {
        cp.add(new ImageCanvas(image),BorderLayout.WEST);
      } // end if
      pack();
      center(parent);
      setResizable(false);
    } // end constructor CustomDlg

    protected void selectFile(int index) {
      File file = new File(((TextField)comps[index]).getText());
      String name,path;
      FileDialog dlg;
      File result = null;

      if (!file.isAbsolute())
        file = new File(getProjectDirectory(),file.getPath());
      // end if
      if (file.isDirectory()) {
        name = "";
        path = file.getAbsolutePath();
      }
      else {
        name = file.getName();
        path = file.getParent();
      } // end if
      dlg = new FileDialog(curFrame,items[index].desc,FileDialog.LOAD);
      dlg.setDirectory(path);
      dlg.setFile(name);
      dlg.show();
      name = dlg.getFile();
      if (name != null) {
        path = dlg.getDirectory();
        if (path == null) result = new File(name);
          else result = new File(path,name);
      } // end if
      if (result == null) return;
      path = result.getAbsolutePath();
      path = StringUtil.relativePath(path,getProjectDirectory());
      ((TextField)comps[index]).setText(path);
    } // end selectFile

  void readResults() {
    if (isCanceled()) return;
    for (int i=0; i<comps.length; i++) {
      if (comps[i] instanceof Checkbox) {
        Checkbox cb = (Checkbox)comps[i];

        items[i].initVal = new Boolean(cb.getState());
      }
      else {
        Utils.Assert.assert(Utils.Assert.ENABLED &&
          (comps[i] instanceof TextField));
        TextField tf = (TextField)comps[i];

        if (tf.isEditable()) {
          items[i].initVal = tf.getText();
        }
        else {
          File file = new File(tf.getText());

          if (!file.isAbsolute())
            file = new File(getProjectDirectory(),tf.getText());
          // end if
          items[i].initVal = file;
        } // end if
      } // end if
    } // end for
  } // end readResults

  } // end CustomDlg

} // end DefaultMacroprocessor

/* ******************************************************************
   end of file
*********************************************************************/
