/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : ErrorMsgPanel
   date       : 15.07.98
   description: a panel that displays error messages from the
                compiler.

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.URL;

import javax.swing.*;
import javax.swing.text.TextAction;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import Utils.ResourceUtil;
import Messages.ErrorMessage;
import Messages.LocatableErrorMessage;
import Messages.ClearErrorMessage;

/* ******************************************************************
   top level class declaration
*********************************************************************/

class ErrorMsgPanel extends JPanel
  implements ListSelectionListener, Observer, Runnable {

  // class variable declarations
  /** Key to the error messages panel label text */
  public static final String ERRORMSG_LABEL = "elLabelText";
  /** Key to the error list source label resource */
  private static final String SOURCE_LABEL = "pSourceLabel";

  // instance variable declarations
  /** The parent frame of this panel. */
  private Frame frame;
  private ResourceBundle resources;
  private Properties project;
  /** The Action to perform on a mouse double click. */
  private Action action;
  private String labelText;
  private String sourcePrefix;
  private JLabel label;
  private JScrollPane scroller;
  private DefaultListModel model    = new DefaultListModel();
  private DefaultListModel tmpModel = new DefaultListModel();
  private JList list                = new JList(model);
  /** The current selected list index or -1 if nothing is selected. */
  private int selectedIndex = -1;
  /** Holds the previous cursor while displaying the wait cursor. */
  private Cursor oldCursor;

  // constructors
  ErrorMsgPanel(Frame frame, ResourceBundle resources, Properties project,
                Action action) {
    this.frame     = frame;
    this.resources = resources;
    this.project   = project;
    this.action    = action;
    setLayout(new BorderLayout());
    labelText     = ResourceUtil.getResStringOrKey(resources,ERRORMSG_LABEL);
    sourcePrefix  = ResourceUtil.getResStringOrKey(resources,SOURCE_LABEL);
    sourcePrefix += " ";
    label         = new JLabel(labelText);
    add("North",label);
    scroller = new JScrollPane(list);
    URL url = ResourceUtil.getResource(this,this.resources,Const.ERROR_IMAGE);
    ImageIcon errImage = url == null ? null : new ImageIcon(url);
    url = ResourceUtil.getResource(this,this.resources,Const.WARNING_IMAGE);
    ImageIcon warnImage = url == null ? null : new ImageIcon(url);
    url = ResourceUtil.getResource(this,this.resources,Const.SUCCESS_IMAGE);
    ImageIcon succImage = url == null ? null : new ImageIcon(url);
    list.setCellRenderer(new ErrorCellRenderer(errImage,warnImage,succImage));
    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    MouseListener mouseListener = new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if ((e.getClickCount() == 2) && !list.isSelectionEmpty()) {
          Object obj = list.getSelectedValue();

          if (obj instanceof LocatableErrorMessage) {
            startAction();
          }
          else {
            Toolkit.getDefaultToolkit().beep();
          } // end if
        } // end if
      } // end mouseClicked
    }; // end anonymous class
    list.addMouseListener(mouseListener);
    list.addListSelectionListener(this);
    add("Center",scroller);
  } // end constructor ErrorMsgPanel

  // methods
  /** Start the action which should be invoked after double clicks. */
  protected void startAction() {
    if ((action != null) && action.isEnabled()) {
      oldCursor = frame.getCursor();
      frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      list.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      frame.getToolkit().sync();
      SwingUtilities.invokeLater(this);
    }
    else {
      Toolkit.getDefaultToolkit().beep();
    } // end if
  } // end startAction

  //---- Runnable --------------------------------------------------
  public void run() {
    LocatableErrorMessage em;
    ActionEvent ae;

    em = (LocatableErrorMessage)list.getSelectedValue();
    ae = new ActionEvent(this,ActionEvent.ACTION_PERFORMED,
                         (em.line - 1) + " " + em.file.getAbsolutePath());

    action.actionPerformed(ae);
    frame.setCursor(oldCursor);
    list.setCursor(oldCursor);
    frame.getToolkit().sync();
  } // end run

  //--------- ListSelectionListener ---------------------------------
  public void valueChanged(ListSelectionEvent e) {
    int index = list.isSelectionEmpty() ? -1 : list.getSelectedIndex();
    Object obj;
    String str;

    if (index == selectedIndex) return;
    obj = list.getSelectedValue();

    if (obj == null) {
      list.setToolTipText(null);
      label.setToolTipText(null);
      return;
    } // end if
    str = ((ErrorMessage)obj).getPopupInfo();
    if ((str != null) && (obj instanceof LocatableErrorMessage)) {
      list.setToolTipText(sourcePrefix + str);
      label.setToolTipText(sourcePrefix + str);
    }
    else {
      list.setToolTipText(str);
      label.setToolTipText(str);
    } // end if
  } // end valueChanged

  //--------- Observer ----------------------------------------------
  public void update(Observable o, Object arg) {
    if (arg instanceof ErrorMessage) {
      ErrorMessage errorMsg = (ErrorMessage)arg;

      if (errorMsg instanceof ClearErrorMessage) {
        list.setToolTipText(null);
        label.setToolTipText(null);
        model.removeAllElements();
      }
      else {
        model.addElement(errorMsg);
      } // end if
    } // end if
  } // end update

  // inner classes
  class ErrorCellRenderer extends JLabel implements ListCellRenderer {

    // instance variable declarations
    private boolean focused = false;
    private Icon errIcon;
    private Icon warnIcon;
    private Icon succIcon;

    // constructors
    public ErrorCellRenderer (Icon errIcon, Icon warnIcon, Icon succIcon) {
      this.errIcon  = errIcon;
      this.warnIcon = warnIcon;
      this.succIcon = succIcon;
      setOpaque(true);
    } // end constructor ErrorCellRenderer

    public Component getListCellRendererComponent(
        JList list, Object value, int index,
        boolean isSelected, boolean cellHasFocus) {
      if (value == null) {
        setText("");
        setIcon(null);
      }
      else { // value instance of ErrorMessage
        ErrorMessage em = (ErrorMessage)value;

        setText(em.getDescription());
        switch (em.getPriority()) {
          case ErrorMessage.MESSAGE: setIcon(succIcon); break;
          case ErrorMessage.WARNING: setIcon(warnIcon); break;
          default: /* if error */    setIcon(errIcon);
        } // end switch
      } // end if
      setBackground (isSelected ?
        SystemColor.textHighlight :
        SystemColor.window);
      setForeground (isSelected ?
        SystemColor.textHighlightText :
        SystemColor.windowText);
      return this;
    } // end getListCellRendererComponent

    public Dimension getPreferredSize() {
      Dimension dim = super.getPreferredSize();
      dim.width += 15;  // widen
      return dim;
    } // end getPreferredSize

  } // end ErrorCellRenderer

} // end ErrorMsgPanel

/* ******************************************************************
   end of file
*********************************************************************/
