/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : FontDlg
   date       : 04.05.98
   description: a dialog that allows to choose a font

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*; 
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class FontDlg extends Dialog
  implements ActionListener,ItemListener {

  // class variable declarations
  private final static String TITLE_PREFIX  = "title";
  private final static String TITLE         = TITLE_PREFIX + "Font";
  private final static String LABEL_PREFIX  = "label";
  private final static String BUTTON_OK     = LABEL_PREFIX + "OK";
  private final static String BUTTON_CANCEL = LABEL_PREFIX + "Cancel";
  private final static String LABEL_FONT    = LABEL_PREFIX + "FontName";
  private final static String LABEL_SIZE    = LABEL_PREFIX + "Size";
  private final static String CHECKBOX_ITAL = LABEL_PREFIX + "Italic";
  private final static String CHECKBOX_BOLD = LABEL_PREFIX + "Bold";
  private final static String DEF_FONT      = "Courier";
  private final static int    DEF_SIZE      = 12;
  private final static String TEST_TEXT     = "AaBbYyZz";

  private final static int[] FONT_SIZE = {
    8 ,9 ,10,
    11,12,14,
    16,18,20,
    24,36,48
  };

  // instance variable declarations
  private Frame parent;
  private Font curFont;
  private ResourceBundle resources;
  private String resourcePrefix;
  private boolean canceled;
  private Button okButton;
  private Button cancelButton;
  private java.awt.List fontList;
  private java.awt.List sizeList;
  private Checkbox italicCheckbox;
  private Checkbox boldCheckbox;
  private FontCanvas fontCanvas;

  // constructors
  /**
   * Create a initially modal font dialog box.
   *
   * @param parent The parent frame.
   */
  public FontDlg(Frame parent) {
    this(parent,new Font(DEF_FONT,Font.PLAIN,DEF_SIZE),null,"");
  } // end constructor FontDlg

  /**
   * Create a initially modal font dialog box.
   *
   * @param parent The parent frame.
   * @param font The specified font.
   */
  public FontDlg(Frame parent, Font font) {
    this(parent,font,null,"");
  } // end constructor FontDlg

  /**
   * Create a initially modal font dialog box.
   *
   * @param parent The parent frame.
   * @param font The specified font.
   * @param resources A resource bundle to look for widget labels
   */
  public FontDlg(Frame parent, Font font, ResourceBundle resources) {
    this(parent,font,resources,"");
  } // end constructor FontDlg

  /**
   * Create a initially modal font dialog box.
   *
   * @param parent The parent frame.
   * @param font The specified font.
   * @param resources A resource bundle to look for widget labels
   * @param resourcePrefix A string that acts as prefix for resource keys.
   */
  public FontDlg(Frame parent, Font font, ResourceBundle resources,
                   String resourcePrefix) {
    super(parent,true);

    this.parent         = parent;
    curFont             = font;
    this.resources      = resources;
    this.resourcePrefix = resourcePrefix;

    setTitle(getTitleResource(TITLE));
    // add a window listener that can handle the window close event
    DlgCloser dlgCloser = new DlgCloser();
    addWindowListener(dlgCloser);
    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();
    setLayout(gridbag);
    // create the widgets and do the layout
    Label fontLabel        = new Label(getLabelResource(LABEL_FONT));
    constraints.insets     = new Insets(10,6,0,12);
    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.WEST; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    constraints.gridheight = 1; 
    gridbag.setConstraints(fontLabel,constraints); 
    add(fontLabel);

    Label sizeLabel       = new Label(getLabelResource(LABEL_SIZE));
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(sizeLabel,constraints); 
    add(sizeLabel);

    fontList = new java.awt.List(5);
    String fontNames[] = getToolkit().getFontList();
    for (int i=0; i<fontNames.length; i++) {
      fontList.add(fontNames[i]);
      if (fontNames[i].equals(curFont.getName())) fontList.select(i);
    } // end for
    fontList.addItemListener(this);
    constraints.insets    = new Insets(0,6,0,12);
    constraints.anchor    = GridBagConstraints.CENTER; 
    constraints.gridwidth = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(fontList,constraints); 
    add(fontList);

    sizeList = new java.awt.List(5);
    for (int i=0; i<FONT_SIZE.length; i++) {
      sizeList.add(""+FONT_SIZE[i]);
      if (FONT_SIZE[i] == curFont.getSize()) sizeList.select(i);
    } // end if
    sizeList.addItemListener(this);
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(sizeList,constraints); 
    add(sizeList);

    italicCheckbox = new Checkbox(getLabelResource(CHECKBOX_ITAL));
    italicCheckbox.setState((curFont.getStyle() & Font.ITALIC) != 0); 
    italicCheckbox.addItemListener(this);

    boldCheckbox = new Checkbox(getLabelResource(CHECKBOX_BOLD));
    boldCheckbox.setState((curFont.getStyle() & Font.BOLD) != 0); 
    boldCheckbox.addItemListener(this);

    Panel checkboxPanel = new Panel();
    checkboxPanel.setLayout(new GridLayout(2,1,10,0));
    checkboxPanel.add(italicCheckbox);
    checkboxPanel.add(boldCheckbox);
    constraints.insets     = new Insets(10,6,0,12);
    constraints.anchor     = GridBagConstraints.WEST;
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    constraints.gridheight = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(checkboxPanel,constraints); 
    add(checkboxPanel);

    fontCanvas = new FontCanvas(curFont,TEST_TEXT);
    constraints.fill      = GridBagConstraints.HORIZONTAL;
    constraints.anchor    = GridBagConstraints.CENTER;
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(fontCanvas,constraints); 
    add(fontCanvas);

    okButton = new Button(getLabelResource(BUTTON_OK));
    okButton.addActionListener(this);

    cancelButton = new Button(getLabelResource(BUTTON_CANCEL));
    cancelButton.addActionListener(dlgCloser);

    Panel buttonPanel = new Panel();
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    constraints.insets     = new Insets(10,6,10,6);
    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.gridheight = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(buttonPanel,constraints); 
    add(buttonPanel);

    pack();
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation((screen.width  - getSize().width ) / 2,
                (screen.height - getSize().height) / 2);
    setResizable(false);
  } // end constructor FontDlg

  // methods
  /**
   * Returns true if dialog has been canceled
   */
  public boolean isCanceled() {
    return canceled;
  } // end isCanceled

  /**
   * Returns the currently selected font.
   */
  public Font getSelectedFont() {
    return curFont;
  } // end getSelectedFont

  public void actionPerformed(ActionEvent e) {
    canceled = false;
    setVisible(false);
    dispose();
  } // end actionPerformed

  public void itemStateChanged(ItemEvent e) {
    int style = italicCheckbox.getState() ? Font.ITALIC : 0;
    int size  = sizeList.getSelectedIndex();

    if (boldCheckbox.getState()) style = style | Font.BOLD;
    if (size < 0) size = curFont.getSize();
      else size = FONT_SIZE[size];
    curFont = new Font(fontList.getSelectedItem(),style,size);
    fontCanvas.makePreview(curFont);
  } // end itemStateChanged

  /**
   * Returns a resource string from the resource bundle or null,
   * if it doesn't exist.
   */
  protected String getStringResource(String key) {
    String str;

    if (resources != null) try {
      str = resources.getString(resourcePrefix+key);
    } catch (MissingResourceException excep) {
	    str = null;
    }
    else
      str = null;
    // end if
    return str;
  } // end getStringResource

  /**
   * Returns a label string from the resource bundle or a default
   * value, if it doesn't exist.
   */
  protected String getLabelResource(String key) {
    String str = getStringResource(key);

    if (str == null) return key.substring(LABEL_PREFIX.length());
      else return str;
  } // end getLabelResource

  /**
   * Returns a title string from the resource bundle or a default
   * value, if it doesn't exist.
   */
  protected String getTitleResource(String key) {
    String str = getStringResource(key);

    if (str == null) return key.substring(TITLE_PREFIX.length());
      else return str;
  } // end getTitleResource

  // inner classes
  class DlgCloser extends WindowAdapter implements ActionListener {

    // methods
    public void windowActivated(WindowEvent e) {
      fontList.requestFocus();
    } // end WindowActivated

    public void windowClosing(WindowEvent e) {
      close(e);
    } // end windowClosing

    public void actionPerformed(ActionEvent e) {
      close(e);
    } // end actionPerformed

    protected void close(AWTEvent e) {
      canceled = true;
      setVisible(false);
      dispose();
    } // end close

  } // end DlgCloser

  /**
   * Allows to give a preview to a selected font.
   */
  class FontCanvas extends Canvas {

    // instance variable declarations
    private Font curFont;
    private String text;

    // constructors
    public FontCanvas(Font font, String text) {
      super();
      curFont   = font;
      this.text = text;
      setBackground(SystemColor.window);
      setForeground(SystemColor.windowText);
    } // end constructor FontCanvas

    // methods
    /** Set the text displayed by this canvas. */
    public void setText(String text) {
      this.text = text;
      repaint();
    } // end setText

    /** Draws a preview of the spezified font. */
    public void makePreview(Font font) {
      curFont = font;
      repaint();
    } // end makePreview

    /** This method is called to repaint this canvas. */
    public void paint(Graphics g) {
      Dimension d = getSize();
      FontMetrics fm;

      g.setColor(getForeground());
      g.setFont(curFont);
      fm = g.getFontMetrics();
      g.drawString(text,2,d.height / 2 +
                   (fm.getAscent() + fm.getDescent()) / 2);
      g.setColor(SystemColor.control);
      g.draw3DRect(0,0,d.width,d.height,false);
    } // end paint

    /** Gets the mininimum size of this component. */
    public Dimension getMinimumSize() {
      return new Dimension(50,50);
    } // end getMinimumSize

    /** Gets the preferred size of this component. */
    public Dimension getPreferredSize() {
      return getMinimumSize();
    } // end getPreferredSize

  } // end FontCanvas

} // end FontDlg

/* ******************************************************************
   end of file
*********************************************************************/
