/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : HtmlPanel
   date       : 20.05.98
   description: a panel for viewing HTML documents, and following
                their links

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.io.IOException;
import java.util.Vector;

/* ******************************************************************
   top level class declaration
*********************************************************************/

/**
 * @version 1.0 20/05/98
 * @author Edgar Puehringer
 */
public class HtmlPanel extends JPanel implements HyperlinkListener {

  // class variable declarations
  public final static String URL_BACKWARD_ACTION = "url-backward";
  public final static String URL_FORWARD_ACTION  = "url-forward";
  public final static String SCROLL_HOME_ACTION  = "scroll-home";
  public final static String DEF_PAGE_ACTION     = "def-page";

  public final static int DEF_URL_LIST_SIZE = 5;

  // instance variable declarations
  private JEditorPane html;
  private JScrollPane scroller;
  private URLTracker tracker;
  private URL curPage;
  private URL[] defaultURLs;
  /** Used by a inner class that reports an error. */
  private IOException tmp;

  private URLBackwardAction urlBackwardAction = new URLBackwardAction();
  private URLForwardAction  urlForwardAction  = new URLForwardAction();

  /** Actions defined by the HtmlPanel class. */
  private Action[] defaultActions = {
    urlBackwardAction,
    urlForwardAction,
    new ScrollHomeAction()
  };

  /** Actions to reach the default pages. */
  private Action[] defPageActions;

  // constructors
  /**
   * Create a new html panel.
   *
   * @param url The url of the page to display.
   */
  public HtmlPanel(URL url) {
    this(url,URLTracker.DEF_CAPACITY);
  } // end constructor HtmlPanel

  /**
   * Create a new html panel.
   *
   * @param url The url of the page to display.
   * @param trackLength The maximal length to track by the back/forward action.
   */
  public HtmlPanel(URL url, int trackLength) {
    this(url,trackLength,DEF_URL_LIST_SIZE);
  } // end constructor HtmlPanel

  /**
   * Create a new html panel.
   *
   * @param url The url of the page to display.
   * @param trackLength The maximal length to track by the back/forward action.
   * @param urlListSize The length of predefined url list.
   */
  public HtmlPanel(URL url, int trackLength, int urlListSize) {
    tracker = new URLTracker(trackLength);
    urlBackwardAction.setEnabled(tracker.canGoBackward());
    urlForwardAction.setEnabled(tracker.canGoForward());
    setLayout(new BorderLayout());
    try {
      html = new JEditorPane(url);
      tracker.setURL(url);
    } catch (IOException excep) {
      html = new JEditorPane();
      tmp  = excep;
      SwingUtilities.invokeLater(new Thread() {

        public void run() {
          reportError(tmp);
          tmp = null;
        } // end run

      });
    }
    curPage = url;
    html.setEditable(false);
    html.setBackground(SystemColor.info);
    html.setForeground(SystemColor.infoText);
    html.addHyperlinkListener(this);
    scroller = new JScrollPane();

    JViewport vp = scroller.getViewport();

    vp.add(html);
    vp.setBackingStoreEnabled(true);
    add(scroller,BorderLayout.CENTER);
    defaultURLs = new URL[urlListSize];
    defPageActions = new Action[urlListSize];
    for (int i=0; i<urlListSize; i++) {
      defaultURLs[i]    = null;
      defPageActions[i] = new DefPageAction(i);
    } // end for
  } // end constructor HtmlPanel

  // methods
  /** Set the page to display. */
  public void setPage(URL u) {
    if (u.equals(html.getPage())) return;
    Cursor c          = html.getCursor();
    Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);

    html.setCursor(waitCursor);
    SwingUtilities.invokeLater(new PageLoader(u,c));
  } // end setPage

  /** Set a default page. */
  public void setDefaultPage(URL u,int index) {
    defaultURLs[index] = u;
  } // end setDefaultPage

  /**
   * Fetch the list of actions supported by this
   * editor.  It is implemented to return the list
   * of actions supported by the embedded JTextComponent
   * augmented with the actions defined locally.
   */
  public Action[] getActions() {
    return TextAction.augmentList(html.getActions(),
             TextAction.augmentList(defaultActions,defPageActions));
  } // end getActions

  /**
   * Notification of a change relative to a
   * hyperlink.
   */
  public void hyperlinkUpdate(HyperlinkEvent e) {
    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
      linkActivated(e.getURL());
    } // end if
  } // end hyperLinkUpdate

  /**
   * Follows the reference in an
   * link.  The given url is the requested reference.
   * By default this calls setPage,
   * and if an exception is thrown the original previous
   * document is restored and a beep sounded.  If an
   * attempt was made to follow a link, but it represented
   * a malformed url, this method will be called with a
   * null argument.
   *
   * @param u the URL to follow
   */
  protected void linkActivated(URL u) {
    Cursor c          = html.getCursor();
    Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);

    html.setCursor(waitCursor);
    SwingUtilities.invokeLater(new PageLoader(u,c));
  } // end linkActivated

  /** Prints the stack trace for an exception. */
  protected void reportError(IOException excep) {
    excep.printStackTrace();
  } // end reportError

  // inner classes
  /**
   * Temporary class that loads synchronously (although
   * later than the request so that a cursor change
   * can be done).
   */
  class PageLoader implements Runnable {

    // instance variable declarations
    private URL url;
    private Cursor cursor;

    // constructors
    PageLoader(URL u, Cursor c) {
      url    = u;
      cursor = c;
    } // end constructor PageLoader

    // methods
    public void run() {
      if (url == null) {
        Container parent = html.getParent();

        // restore the original cursor
        html.setCursor(cursor);
        parent.repaint();
      } else {
        Document doc = html.getDocument();

        try {
          URL u = url;

          html.setPage(url);
          tracker.setURL(u);
          urlBackwardAction.setEnabled(tracker.canGoBackward());
          urlForwardAction.setEnabled(tracker.canGoForward());
        } catch (IOException excep) {
          html.setDocument(doc);
          getToolkit().beep();
          reportError(excep);
        } finally {
          // schedule the cursor to revert after the paint has happended.
          url = null;
          SwingUtilities.invokeLater(this);
        }
      } // end if
    } // end run

  } // end PageLoader

  class URLBackwardAction extends AbstractAction {

    // constructors
    URLBackwardAction() { super(URL_BACKWARD_ACTION); }

    // instance methods
    public void actionPerformed(ActionEvent e) {
      Cursor c          = html.getCursor();
      Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
      URL u;

      if (!tracker.canGoBackward()) {
        Toolkit.getDefaultToolkit().beep();
        return;
      } // end if
      html.setCursor(waitCursor);
      u = tracker.getPreviousURL();
      SwingUtilities.invokeLater(new PageLoader(u,c));
    } // end actionPerformed

  } // end URLBackwardAction

  class URLForwardAction extends AbstractAction {

    // constructors
    URLForwardAction() { super(URL_FORWARD_ACTION); }

    // instance methods
    public void actionPerformed(ActionEvent e) {
      Cursor c          = html.getCursor();
      Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
      URL u;

      if (!tracker.canGoForward()) {
        Toolkit.getDefaultToolkit().beep();
        return;
      } // end if
      html.setCursor(waitCursor);
      u = tracker.getNextURL();
      SwingUtilities.invokeLater(new PageLoader(u,c));
    } // end actionPerformed

  } // end URLForwardAction

  class ScrollHomeAction extends AbstractAction {

    // constructors
    ScrollHomeAction() { super(SCROLL_HOME_ACTION); }

    // instance methods
    public void actionPerformed(ActionEvent e) {
      JScrollBar vsb = scroller.getVerticalScrollBar();
      JScrollBar hsb = scroller.getHorizontalScrollBar();

      if (vsb != null) vsb.setValue(vsb.getMinimum());
      if (hsb != null) hsb.setValue(hsb.getMinimum());
    } // end actionPerformed

  } // end ScrollHomeAction

  /** Loads one of the default pages. */
  class DefPageAction extends AbstractAction {

    // instance variable declarations
    protected int index;

    // constructors
    DefPageAction(int index) {
      super(DEF_PAGE_ACTION + (index + 1));
      this.index = index;
    } // end constructor DefPageAction

    // instance methods
    public void actionPerformed(ActionEvent e) {
      if (defaultURLs[index] != null) setPage(defaultURLs[index]);
    } // end actionPerformed

  } // end DefPageAction

} // end HtmlPanel

/* ******************************************************************
   classes
*********************************************************************/

/**
 * The URLTracker class follows the hyperlinks of hypertextbrowsers
 * and provides forward/back operation.
 *
 * @author puehringer edgar
 */
class URLTracker extends Vector {

  // class variable declarations
  /** The default capacity of the url tracker. */
  public final static int DEF_CAPACITY = 10;

  // instance variable declarations
  /** The capacity of the url tracker. */
  private int capacity;
  /** The position of the current url in the vector. */
  private int trackPos;

  // constructors
  /** Create a new url tracker with default capacity. */
  public URLTracker() {
    this(DEF_CAPACITY);
  } // end constructor URLTracker

  /** Create a new url tracker with the specified capacity. */
  public URLTracker(int capacity) {
    super(capacity + 1);
    this.capacity = capacity;
    trackPos      = -1;
  } // end constructor URLTracker

  // methods
  /** Returns the previous url. */
  public URL getPreviousURL() {
    if (canGoBackward()) return (URL)elementAt(trackPos - 1);
    return null;
  } // end getPreviousURL

  /** Returns the current url. */
  public URL getCurrentURL() {
    if (trackPos < 0) return null;
    return (URL)elementAt(trackPos);
  } // end getCurrentURL

  /** Returns the next url. */
  public URL getNextURL() {
    if (canGoForward()) return (URL)elementAt(trackPos + 1);
    return null;
  } // end getNextURL

  /** Sets the previous page to be the current page. */
  public void goBackward() {
    trackPos--;
  } // end goBackward

  /** Sets the next page to be the current page. */
  public void goForward() {
    trackPos++;
  } // end goForward

  /** Returns true if a previous page is available. */
  public boolean canGoBackward() {
    return trackPos > 0;
  } // end canGoBackward

  /** Returns true if a next page is available. */
  public boolean canGoForward() {
    return trackPos < size() - 1;
  } // end canGoForward

  /** Adds an url to the url tracker. */
  public void setURL(URL url) {
    if (url == null) return;
    // check if url matches previous url
    if (canGoBackward() && getPreviousURL().equals(url)) {
      trackPos--;
      return;
    } // end if
    // check if url matches next url
    if (canGoForward() && getNextURL().equals(url)) {
      trackPos++;
      return;
    } // end if
    // check if url matches current url
    URL tmp = getCurrentURL();

    if ((tmp != null) && tmp.equals(url)) return;
    // size limit
    if (trackPos >= capacity) removeElementAt(0);
      else trackPos++;
    insertElementAt(url,trackPos);
    // clip all elements after the current position
    setSize(trackPos + 1);
  } // end setURL

} // end URLTracker

/* ******************************************************************
   end of file
*********************************************************************/
