/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : JMenuLoader
   date       : 20.04.98
   description: A class that builds up a menu by reading the
                settings from a resource file.

*********************************************************************/

package MenuPreview;

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import javax.swing.event.*;
import javax.swing.*;

import Utils.ResourceUtil;

/* ******************************************************************
   top level class declaration
*********************************************************************/

/**
 * A class that builds up a menu by reading the settings from a
 * resource file.
 *
 * @author  Puehringer Edgar
 * @version 1.0 04/20/98
 */
class JMenuLoader {

  // class variable declarations
 /** The char that forms the mnemonic in the label string. */
  private static Class eventClass;
  private static Class keyEventClass;

  // instance variable declarations
  private Properties resources;
  private Hashtable commands;
  private ActionListener defaultAction;
  private String imagePath;
  private boolean useImages;

  // static initializers
  static {
    try {
      keyEventClass = Class.forName("java.awt.event.KeyEvent");
    } catch (ClassNotFoundException e) { e.printStackTrace(); }
    try {
      eventClass = Class.forName("java.awt.Event");
    } catch (ClassNotFoundException e) { e.printStackTrace(); }
  } // end static initializer

  // constructors
  JMenuLoader(InputStream is) throws IOException {
    resources = new Properties();
    resources.load(is);
    commands      = null;
    defaultAction = null;
    imagePath     = System.getProperty("user.dir");
    useImages     = true;
  } // end constructor JMenuLoader

  JMenuLoader(Properties resources) {
    this.resources = resources;
    commands       = null;
    defaultAction  = null;
    imagePath      = System.getProperty("user.dir");
    useImages      = true;
  } // end constructor JMenuLoader

  /**
   * Create a new JMenuLoader object.  If the imagePath parameter
   * is null, no images are displayed.
   */
  JMenuLoader(Properties resources, String imagePath) {
    this.resources = resources;
    this.imagePath = imagePath;
    commands       = null;
    defaultAction  = null;
    useImages      = (imagePath != null);
  } // end constructor JMenuLoader

  // class methods
  /** Filters the mnemonic identifiers out of a label string. */
  private static String filterMnemonic(String label, char[] ch) {
    int index = label.indexOf(Const.MNEMONIC_ESCAPE);

    if (index < 0) return label;
    if ((index + 1) < label.length()) {
      char c = label.charAt(index + 1);

      if (c != Const.MNEMONIC_ESCAPE) ch[0] = c;
      return label.substring(0,index) + c +
        filterMnemonic(label.substring(index + 2),ch);
    }
    else {
      return label.substring(0,index);
    } // end if
  } // end filterMnemonic

  // instance methods
  /** Return a hashtable containing the menu commands. */
  public Hashtable getCommands() {
    return commands;
  } // end getCommands

  /**
   * Set the hashtable containing the menu commands.
   * If you use it, this must be called before createMenuBar.
   *
   * @see #createMenuBar
   */
  public void setCommands(Hashtable commands) {
    this.commands = commands;
  } // end setCommands

  /**
   * Return the default action listener.
   */
  public ActionListener getDefaultActionListener() {
    return defaultAction;
  } // end getDefaultActionListener

  /**
   * Set the default action listener.
   * If you use it, this must be called before createMenuBar.
   *
   * @see #createMenuBar
   */
  public void setDefaultActionListener(ActionListener al) {
    this.defaultAction = al;
  } // end setDefaultActionListener

  /** This is the hook through which all menu items are created. */
  public JMenuItem createMenuItem(String key) {
    URL url      = useImages ? getResource(key + Const.IMAGE_SUFFIX) : null;
    String label = getResourceString(key + Const.LABEL_SUFFIX);
    String type  = getResourceString(key + Const.TYPE_SUFFIX);
    String astr  = getResourceString(key + Const.ACTION_SUFFIX);
    String acc   = getResourceString(key + Const.ACCEL_SUFFIX);
    JMenuItem mi;
    ActionListener a;
    boolean isSubmenu;
    char[] mnemonic = new char[1];

    // prepare the menu item label
    if (label == null) {
      reportError("Label for menu item key \"" + key + "\" not found");
      return null;
    }
    else {
      label = filterMnemonic(label,mnemonic);
    } // end if
    // create a menu item matching the spezified type
    if (type != null) type = type.toLowerCase();
    if (astr == null) astr = key;
    if ((type == null) || type.equals(Const.TYPE_ITEM)) {
      mi = new JMenuItem(label);
      mi.setActionCommand(astr);
      a = getAction(astr);
      isSubmenu = false;
    } else if (type.equals(Const.TYPE_CHECKBOX)) {
      mi = new JCheckBoxMenuItem(label);
      mi.setActionCommand(astr);
      a = getAction(astr);
      isSubmenu = false;
    } else if (type.equals(Const.TYPE_MENU)) {
      mi        = createMenu(key);
      a         = null;
      isSubmenu = true;
    } else {
      reportError("Unknown menu type: " + type);
      return null;
    } // end if
    // add an icon
    if (url != null) {
      mi.setHorizontalTextPosition(JButton.RIGHT);
      mi.setIcon(loadImageIcon(url));
    } // end if
    // connect an action to the menu command
    if (a != null) {
      mi.addActionListener(a);
      if (a instanceof Action) {
        Action action = (Action)a;
        action.addPropertyChangeListener(new ActionChangedListener(mi));
        mi.setEnabled(action.isEnabled());
      } // end if
    } else if (!isSubmenu) {
      mi.setEnabled(false);
    } // end if
    // set the accelerator
    if (acc != null) {
      try {
        String[] arr = tokenize(acc);
        int value    = keyEventClass.getField(arr[0]).getInt(null);
        int mod      = Event.CTRL_MASK;
        for (int i=1; i<arr.length; i++) {
          mod ^= eventClass.getField(arr[i]).getInt(null);
        } // end for
        mi.setAccelerator(KeyStroke.getKeyStroke(value,mod));
      } catch (NoSuchFieldException e1) {
        reportError("Unknown keycode: " + acc);
      } catch (SecurityException e2) {
        e2.printStackTrace();
      } catch (IllegalAccessException e3) {
        e3.printStackTrace();
      }
    } // end if
    // set the mnemonic
    if (mnemonic[0] != (char)0) mi.setMnemonic(mnemonic[0]);
    mi.setBackground(SystemColor.menu);
    return mi;
  } // end createMenuItem

  /**
   * Return the Action stored in the command hashtable.  If it doesn't
   * exist, return the default action.
   */
  protected ActionListener getAction(String cmd) {
    ActionListener a;
    if (commands == null) a = null;
      else a = (ActionListener)commands.get(cmd);
    if (a == null) return defaultAction;
      else return a;
  } // end getAction

  /**
   * Read a string from the resource properties containing the menu
   * definition.
   */
  protected String getResourceString(String key) {
    return resources.getProperty(key);
  } // end getResourceString

  protected URL getResource(String key) {
    String name = getResourceString(key);
    String path;
    URL url;

    if(name == null) return null;
    path = ResourceUtil.fileToURL(imagePath);
    if (path.endsWith("/")) path += name;
      else path += ("/" + name);
    try {
      url = new URL(path);
    } catch (MalformedURLException excep) {
      reportError("Malformed URL: " + path);
      url = null;
    }
    return url;
  } // end getResource

  /**
   * Take the given string and chop it up into a series
   * of strings on whitespace boundries.  This is useful
   * for trying to get an array of strings out of the
   * resource file.
   */
  protected String[] tokenize(String input) {
    Vector v          = new Vector();
    StringTokenizer t = new StringTokenizer(input);
    String cmd[];

    while (t.hasMoreTokens()) v.addElement(t.nextToken());
    cmd = new String[v.size()];
    for (int i=0; i<cmd.length; i++) cmd[i] = (String)v.elementAt(i);
    return cmd;
  } // end tokenize

  /**
   * Create the menubar for the app. By default this pulls the
   * definition of the menu from the associated resource file.
   */
  protected JMenuBar createMenuBar(String key) {
    JMenuBar mb   = new JMenuBar();
    String mbDesc = getResourceString(key);
    String[] menuKeys;

    if (mbDesc == null) {
      reportError("MenuBar \"" + key + "\" not found or empty");
      return null;
    } // end if
    menuKeys = tokenize(mbDesc);
    for (int i=0; i<menuKeys.length; i++) {
      JMenu m = createMenu(menuKeys[i]);

      if (m != null) mb.add(m);
    } // end for
    mb.setBackground(SystemColor.menu);
    return mb;
  } // end createMenuBar

  /**
   * Create a menu.  By default this pulls the definition
   * of the menu from the associated resource file.
   */
  protected JMenu createMenu(String key) {
    String label    = getResourceString(key + Const.LABEL_SUFFIX);
    String menuDesc = getResourceString(key);
    String[] itemKeys;
    JMenu menu;
    char[] mnemonic = new char[1];

    if (menuDesc == null) {
      reportError("Menu \"" + key + "\" not found or empty");
      return null;
    } // end if
    itemKeys = tokenize(menuDesc);
    // prepare the menu label
    if (label == null) {
      reportError("Label for menu key \"" + key + "\" not found");
      return null;
    } // end if
    label = filterMnemonic(label,mnemonic);
    if (menuDesc == null) {
      reportError("Menu \"" + key + "\" not found or empty");
      return null;
    } // end if
    menu = new JMenu(label);
    if (mnemonic[0] != (char)0) menu.setMnemonic(mnemonic[0]);
    for (int i = 0; i < itemKeys.length; i++) {
      if (itemKeys[i].equals("-")) {
        menu.addSeparator();
      }
      else {
        JMenuItem mi = createMenuItem(itemKeys[i]);

        if (mi != null) menu.add(mi);
      } // end if
    } // end for
    menu.setBackground(SystemColor.menu);
    return menu;
  } // end createMenu

  /** Hook through which ImageIcons are loaded. */
  protected ImageIcon loadImageIcon(URL url) {
    return new ImageIcon(url);
  } // end loadImageIcon

  protected void reportError(String text) {
    System.err.println("JMenuLoader: " + text + ".");
  } // end reportError

  // inner classes
  private class ActionChangedListener implements PropertyChangeListener {

    // class variable declarations
    JMenuItem menuItem;

    // constructors
    ActionChangedListener(JMenuItem mi) {
      super();
      menuItem = mi;
    } // end constructor ActionChangedListener

    // methods
    public void propertyChange(PropertyChangeEvent e) {
      String propertyName = e.getPropertyName();

      if (e.getPropertyName().equals(Action.NAME)) {
        String text = (String)e.getNewValue();

        menuItem.setText(text);
      } else if (propertyName.equals("enabled")) {
        Boolean enabledState = (Boolean)e.getNewValue();

        menuItem.setEnabled(enabledState.booleanValue());
      } else if (propertyName.equals("checked") &&
                 (menuItem instanceof JCheckBoxMenuItem)) {
        boolean checkedState   = ((Boolean)e.getNewValue()).booleanValue();
        JCheckBoxMenuItem cbmi = (JCheckBoxMenuItem)menuItem;

        if (cbmi.getState() != checkedState)
          cbmi.setState(checkedState);
        // end if
      } // end if
    } // end propertyCange

  } // end ActionChangedListener

} // end JMenuLoader

/* ******************************************************************
   end of file
*********************************************************************/
