/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : ToolBarLoader
   date       : 10.10.98

 *********************************************************************/

package MenuPreview;

/* ******************************************************************
   import statements
 *********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.io.IOException;
import java.util.*;

import Utils.ResourceUtil;

/* ******************************************************************
   top level class declaration
 *********************************************************************/

/**
 * A class that builds up a toolbar by reading the settings from an
 * input stream loaded into a properties object.
 *
 * @author  Puehringer Edgar
 */
class ToolBarLoader {

  // instance variable declarations
  private Properties resources;
  private ActionListener defaultAction;

  // constructors
  ToolBarLoader(InputStream is) throws IOException {
    resources = new Properties();
    resources.load(is);
    defaultAction = null;
  } // end constructor ToolBarLoader

  ToolBarLoader(Properties resources) {
     this.resources = resources;
     defaultAction  = null;
  } // end constructor ToolBarLoader

  // class methods
  /** Filters the mnemonic identifiers out of a label string. */
  private static String filterMnemonic(String label) {
    int index = label.indexOf(Const.MNEMONIC_ESCAPE);

    if (index < 0) return label;
    if ((index + 1) < label.length()) {
      char c = label.charAt(index + 1);

      return label.substring(0,index) + c +
        filterMnemonic(label.substring(index + 2));
    }
    else {
      return label.substring(0,index);
    } // end if
  } // end filterMnemonic

  // methods
  /** Return the default action listener. */
  public ActionListener getDefaultActionListener() {
    return defaultAction;
  } // end getDefaultActionListener

  /**
   * Set the default action listener.
   * If you use it, this must be called before createToolBar.
   *
   * @see #createToolBar
   */
  public void setDefaultActionListener(ActionListener al) {
    defaultAction = al;
  } // end setDefaultActionListener

  /**
   * Create a button to go inside of the toolbar panel.
   *
   * @param key The key in the resource file to serve as the basis
   *            of lookups.
   */
  protected Button createToolBarButton(String key) {
    String label = getResourceString(key + Const.LABEL_SUFFIX);
    String astr  = getResourceString(key + Const.ACTION_SUFFIX);
    Button b;

    if (label == null) {
      reportError("Label property for toolbar button key \"" +
                  key + "\" not found");
      return null;
    } // end if
    b = new Button(filterMnemonic(label)) {

      public boolean isFocusTraversable() {
        return false;
      } // end isFocusTraversable

    };
    if(astr == null) astr = key;
    b.setActionCommand(astr);

    ActionListener a = getAction(astr);

    if(a != null) {
      b.addActionListener(a);
      b.setActionCommand(astr);
    }
    else {
      b.setEnabled(false);
    } // end if
    return b;
  } // end createToolBarButton

  /**
   * Return the Action stored in the command hashtable.  If it doesn't
   * exist, return the default action.
   */
  protected ActionListener getAction(String cmd) {
    return defaultAction;
  } // end getAction

  /**
   * Read a string from the resource file containing the toolbar
   * definition.
   */
  protected String getResourceString(String key) {
    return resources.getProperty(key);
  } // end getResourceString

  /**
   * Take the given string and chop it up into a series
   * of strings on whitespace boundries. This is useful
   * for trying to get an array of strings out of the
   * resource file.
   */
  protected String[] tokenize(String input) {
    Vector v = new Vector();
    StringTokenizer t = new StringTokenizer(input);
    String cmd[];

    while(t.hasMoreTokens()) v.addElement(t.nextToken());
    cmd = new String[v.size()];
    for(int i = 0;i < cmd.length;i++)
      cmd[i] = (String)v.elementAt(i);
    // end for
    return cmd;
  } // end tokenize

  /**
   * Create the toolbar.  By default this reads the
   * resource file for the definition of the toolbar.
   */
  protected Panel createToolBar(String key) {
    Panel tb = new Panel(new FlowLayout(FlowLayout.LEFT));
    String tbDesc = getResourceString(key);
    String[] toolKeys;

    if (tbDesc == null) {
      reportError("ToolBar \"" + key + "\" not found or empty");
      return null;
    } // end if
    toolKeys = tokenize(tbDesc);
    for (int i=0; i<toolKeys.length; i++) {
      if(toolKeys[i].equals("-")) {
        tb.add(new Label(Const.STRUT_BLANKS));
      }
      else {
        Component comp = createTool(toolKeys[i]);

        if (comp != null) tb.add(comp);
      } // end if
    } // end for
    tb.setBackground(SystemColor.control);
    return tb;
  } // end createToolBar

  /** Hook through which every toolbar item is created. */
  protected Component createTool(String key) {
    return createToolBarButton(key);
  } // end createTool

  protected void reportError(String text) {
    System.err.println("ToolBarLoader: " + text + ".");
  } // end reportError

} // end ToolBarLoader

/* ******************************************************************
   end of file
 *********************************************************************/
