// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;

// only show public members and methods
class JavaPublicInterface extends InterfaceFilter{

	JavaPublicInterface(){super();}
	JavaPublicInterface(InterfaceFilter wrap){super(wrap);}

	public boolean acceptMethod(MethodInfo mi){
		return (super.acceptMethod(mi) && (mi.isModifierSet("public")));
	}
	
	public boolean acceptAttribute(AttributeInfo ai){
		return (super.acceptAttribute(ai) && (ai.isModifierSet("public")));
	}
}

// only show nonstatic members and methods
class JavaNonStaticInterface extends InterfaceFilter{

	JavaNonStaticInterface(){super();}
	JavaNonStaticInterface(InterfaceFilter wrap){super(wrap);}

	public boolean acceptMethod(MethodInfo mi){
		return (super.acceptMethod(mi) && (!mi.isModifierSet("static")));
	}
	
	public boolean acceptAttribute(AttributeInfo ai){
		return (super.acceptAttribute(ai) && (!ai.isModifierSet("static")));
	}
}

// only show attributes
class JavaAttributesInterface extends InterfaceFilter{

	JavaAttributesInterface(){super();}
	JavaAttributesInterface(InterfaceFilter wrap){super(wrap);}

	public boolean acceptMethod(MethodInfo mi){
		return false;
	}
	
	public boolean acceptAttribute(AttributeInfo ai){
		return super.acceptAttribute(ai);
	}
}

// only show methods
class JavaMethodInterface extends InterfaceFilter{

	JavaMethodInterface(){super();}
	JavaMethodInterface(InterfaceFilter wrap){super(wrap);}

	public boolean acceptMethod(MethodInfo mi){
		return super.acceptMethod(mi);
	}
	
	public boolean acceptAttribute(AttributeInfo ai){
		return false;
	}
}

// concrete class to represent java classes
public class JavaClassInfo extends ClassInfo{

	protected Vector importList;

	static{
		modResolve = modifierPool.javaMods();
		modMax = 10;
	}

	public JavaClassInfo(){
		super();
		importList = new Vector(4);
		importList.addElement(new JavaImport("java.lang", false));
	}

	public JavaClassInfo(String name, String sname){
		super(name, sname);
		importList = new Vector(4);
		importList.addElement(new JavaImport("java.lang", false));
	}

	public void addImport(String packageName, boolean simpleType){
		importList.addElement(new JavaImport(packageName, simpleType));
	}

	public void addImport(JavaImport ji){
		importList.addElement(ji);
	}

	public Enumeration getImportList(){
		return importList.elements();
	}

	public boolean isPublic(){ return isModifierSet("public"); }
	public boolean isPrivate(){ return isModifierSet("private"); }
	public boolean isProtected(){ return isModifierSet("protected"); }

	public String getInterface(){
		StringBuffer retString = new StringBuffer("");
		retString.append(toString() + "{");

		retString.append("\n// class imports\n");
		for (Enumeration e = importList.elements(); e.hasMoreElements();){
			JavaImport ji = (JavaImport)e.nextElement();
			retString.append("    " + ji.importName);
			if (!ji.simpleType) retString.append(".*");
			retString.append(";\n");
		}

		retString.append("\n// class attributes\n");
		for (Enumeration e = attributes(); e.hasMoreElements();){
			JavaAttributeInfo ai = (JavaAttributeInfo)e.nextElement();
			retString.append("    " + ai.toString()+";\n");
		}
		retString.append("\n// methods\n");
		for (Enumeration e = methods(); e.hasMoreElements();){
			JavaMethodInfo mi = (JavaMethodInfo)e.nextElement();
			retString.append("    " + mi.toString()+";\n");
		}
		retString.append("}");
		return retString.toString();
	}

	public String getFilteredInterface(InterfaceFilter f){
		StringBuffer retString = new StringBuffer("");
		retString.append(toString() + "{");

		retString.append("\n// class attributes\n");
		for (Enumeration e = attributes(); e.hasMoreElements();){
			JavaAttributeInfo ai = (JavaAttributeInfo)e.nextElement();
			if (f.acceptAttribute(ai))
				retString.append("    " + ai.toString()+";\n");
		}
		retString.append("\n// methods\n");
		for (Enumeration e = methods(); e.hasMoreElements();){
			JavaMethodInfo mi = (JavaMethodInfo)e.nextElement();
			if (f.acceptMethod(mi))
				retString.append("    " + mi.toString()+";\n");
		}
		retString.append("}");
		return retString.toString();
	}

	public void load(DataInputStream in) throws IOException{
		super.load(in);
		
		int size = in.readShort();
		importList = new Vector(size);
		for (int i=0; i < size; i++){
			JavaImport ji = new JavaImport();
			ji.load(in);
			importList.addElement(ji);
		}
	}

	public void store(DataOutputStream out) throws IOException{
		super.store(out);
		
		out.writeShort(importList.size());
		for (Enumeration e = importList.elements();e.hasMoreElements();){
			JavaImport ji = (JavaImport)e.nextElement();
			ji.store(out);
		}
	}

	public String getPackage(){
		int idx = name.lastIndexOf(".");
		if (idx < 0) return "";
		return name.substring(0, idx);
	}

	public String toString(){
		return name.substring(name.lastIndexOf(".")+1);
	}
}


